@@ -2,7 +2,7 @@ package Amazon::Credentials;
 
 use strict;
 use warnings;
-use 5.012;
+use 5.010;
 
 use parent qw/Class::Accessor::Fast/;
 
@@ -11,7 +11,7 @@ __PACKAGE__->follow_best_practice;
 __PACKAGE__->mk_accessors(
   qw/aws_secret_access_key aws_access_key_id token region
     user_agent profile debug expiration role container order
-    serialized logger timeout
+    serialized logger timeout insecure
     /
 );
 
@@ -40,8 +40,9 @@ use constant DEFAULT_TIMEOUT => 3; ## no critic (ValuesAndExpressions::ProhibitC
 use constant DEFAULT_WINDOW_INTERVAL => 5; ## no critic (ValuesAndExpressions::ProhibitConstantPragma)
 use constant SECONDS_IN_MINUTE => 60; ## no critic (ValuesAndExpressions::ProhibitConstantPragma)
 use constant SECONDS_IN_HOUR => 3600; ## no critic (ValuesAndExpressions::ProhibitConstantPragma)
+use constant INSECURE_MODE => 2; ## no critic (ValuesAndExpressions::ProhibitConstantPragma)
 
-our $VERSION = '1.0.18';
+our $VERSION = '1.0.19';
 
 # we only log at debug level, create a default logger
 {
@@ -65,8 +66,8 @@ sub new {
 
   my $self = $class->SUPER::new($options);
 
-  $self->set_debug( $ENV{DEBUG} // $self->get_debug ) ;
-  
+  $self->set_debug( $ENV{DEBUG} // $self->get_debug );
+
   if ( !$self->get_logger || reftype( $self->get_logger ) ne 'CODE' ) {
     $self->set_logger( bless { debug => $self->get_debug },
       'Amazon::Credentials::Logger' );
@@ -88,8 +89,14 @@ sub new {
     $self->set_order(DEFAULT_SEARCH_ORDER);
   }
 
-  $self->get_logger->debug('order ' . $self->get_order);
-                           
+  $self->get_logger->debug( 'order ' . $self->get_order );
+
+  if ( $self->get_insecure ) {
+    $self->get_logger->debug( "!! CAUTION !!\n"
+        . "!! You are executing in 'insecure' mode !!\n"
+        . "!! Credentials may be exposed in debug messages !!\n" );
+  }
+
   if ( !$self->get_aws_secret_access_key || !$self->get_aws_access_key_id ) {
     $self->set_credentials;
   }
@@ -205,8 +212,8 @@ sub find_credentials {
   );
 
   foreach my $location (@search_order) {
-    $self->get_logger->debug('searching for credentials: ' . $location);
-    
+    $self->get_logger->debug( 'searching for credentials: ' . $location );
+
     if ( $creds_getters{$location} ) {
       $creds = $creds_getters{$location}->();
     }
@@ -321,6 +328,26 @@ sub get_creds_from_file { ## no critic (Subroutines::ProhibitExcessComplexity)
   return $creds;
 } ## end sub get_creds_from_file
 
+sub dump_ini_file {
+  my ( $self, $ini ) = @_;
+
+  my $safe_rsp;
+
+  if ( $self->get_insecure && $self->get_insecure =~ /^2$/xsm ) {
+    $safe_rsp = Dumper [$ini];
+  }
+  elsif ($self->get_insecure) {
+    $safe_rsp = Dumper [$ini];
+    $safe_rsp =~ s/(aws_[^']+)'([^']+)'([^']+)'/$1'$2'...'/gxsm;
+  }
+  else{
+    $safe_rsp = '** configuration contents blocked by insecure setting **';
+  }
+  
+
+  return $safe_rsp;
+} ## end sub dump_ini_file
+
 sub get_creds_from_ini_file {
   my ( $self, $profile ) = @_;
 
@@ -331,7 +358,7 @@ sub get_creds_from_ini_file {
 
   foreach my $config (qw{ .aws/config .aws/credentials }) {
     last if $creds->{source};
-    
+
     my $profile_name = $profile;
 
     $CWD = home;
@@ -339,8 +366,9 @@ sub get_creds_from_ini_file {
     next if !-e $config;
 
     my $ini = Config::Tiny->read($config);
-    $self->get_logger->debug(Dumper($ini));
-    
+
+    $self->get_logger->debug( $self->dump_ini_file($ini) );
+
     if ( $profile eq 'default' ) {
       next if !$ini->{default}->{profile};
 
@@ -387,7 +415,7 @@ sub get_creds_from_ini_file {
         $creds->{source} = $config;
       }
     } ## end if ($section)
-  } ## end foreach my $config (qw{ aws/config .aws/credentials })
+  } ## end foreach my $config (qw{ .aws/config .aws/credentials })
 
   $CWD = $cwd;
 
@@ -451,6 +479,36 @@ sub _iso8601_to_time {
   return $gmtime;
 } ## end sub _iso8601_to_time
 
+sub dump_response {
+  my ( $self, $rsp ) = @_;
+
+  my $safe_rsp;
+
+  if ( $self->get_insecure && $self->get_insecure =~ /^2$/xsm ) {
+    $safe_rsp = $rsp;
+  }
+  elsif ( $self->get_insecure ) {
+    $safe_rsp = {};
+
+    foreach my $k ( keys %{$rsp} ) {
+      if ( $k =~ /content/xsm ) {
+        my $content = $rsp->{$k};
+        $content
+          =~ s/\"(AccessKeyId|Token|SecretAccessKey)\"\s+\:\s+\"[^\"]+\"/\"$1\" : \"...\"/gxsm;
+        $safe_rsp->{$k} = $content;
+      } ## end if ( $k =~ /content/xsm)
+      else {
+        $safe_rsp->{$k} = $rsp->{$k};
+      }
+    } ## end foreach my $k ( keys %{$rsp...})
+  } ## end elsif ( $self->get_insecure)
+  else {
+    $safe_rsp = '** HTTP RESPONSE blocked by insecure setting **';
+  }
+
+  return Dumper [$safe_rsp];
+} ## end sub dump_response
+
 sub get_creds_from_role {
   my ($self) = @_;
 
@@ -477,7 +535,7 @@ sub get_creds_from_role {
 
       my $rsp = $ua->request($req);
 
-      $self->get_logger->debug( Dumper [ "HTTP RESPONSE:\n", $rsp ] );
+      $self->get_logger->debug( $self->dump_response($rsp) );
 
       # if not 200, then get out of Dodge
       last if !$rsp->is_success;
@@ -590,7 +648,7 @@ sub format_credentials {
 
   my $credential_keys = $self->credential_keys;
 
-  return join "", map { sprintf $format, $_, $credential_keys->{$_} }
+  return join q{}, map { sprintf $format, $_, $credential_keys->{$_} }
     keys %{$credential_keys};
 } ## end sub format_credentials
 
@@ -618,7 +676,7 @@ sub get_creds_from_container {
 
       my $rsp = $ua->request($req);
 
-      $self->get_logger->debug( Dumper [ "HTTP RESPONSE:\n", $rsp ] );
+      $self->get_logger->debug( $self->dump_response($rsp) );
 
       # if not 200, then get out of Dodge
       if ( $rsp->is_success ) {
@@ -707,15 +765,17 @@ constructor will not look in other places for credentials.>
 =item container - Task Role
 
 If the process is running in a container, the container may have a
-task role.  We'll look credentials using the container metadata
-service.
+task role.  The class will look for credentials using the container
+metadata service:
 
  http://169.254.170.2/$AWS_CONTAINER_CREDENTIALS_RELATIVE_URI
 
 =item debug
 
 Set to true for verbose troubleshooting information.  Set C<logger> to
-a logger that implements a logging interface (ala C<Log::log4perl>.
+a logger that implements a logging interface (ala
+C<Log::log4perl>. Debug output canalso be enabled by setting the
+environment variable C<DEBUG> to a I<true>.
 
 =item env - Environment
 
@@ -729,8 +789,9 @@ to provide credentials.
  AWS_SESSION_TOKEN
 
 I<Note that when you set the environment variable AWS_PROFILE, the
-order essentially is overridden and we'll look in your credential
-files (F<~/.aws/config>, F<~/.aws/credentials>) for your credentials.>
+order essentially is overridden and the class will ook in your
+credential files (F<~/.aws/config>, F<~/.aws/credentials>) to resolve
+your credentials.>
 
 =item file - Configuration Files
 
@@ -742,14 +803,33 @@ files (F<~/.aws/config>, F<~/.aws/credentials>) for your credentials.>
 
 =back
 
-The class will attempt to find the credentials in either of these two
-files.  You can also specify a profile to use for looking up the
-credentials by passing it into the constructor or setting it the
-environment variable C<AWS_PROFILE>.  If no profile is provided, the
-default credentials or the first profile found is used.
+The class will attempt to resolve credentials by interpretting the
+information in these two files. You can also specify a profile to use
+for looking up the credentials by passing it into the constructor or
+setting it the environment variable C<AWS_PROFILE>.  If no profile is
+provided, the default credentials or the first profile found is used.
 
  my $aws_creds = Amazon::Credentials->new({ order => [qw/environment role file/] });
 
+=item insecure
+
+DEBUG mode can be enabled to display information that may aid in
+troubleshooting, however output may include credentials.  This
+attribute prevents accidental exfiltration of credentials during
+troubleshooting.  The default setting of C<insecure> is therefore
+false. This will prevent debug messages that may contain credentials
+(HTTP response, configuration file contents) from exposing sensitive
+data.
+
+Set the value to 1 to enable all debug output B<except> the content of
+credentials in HTTP responses. Set the value to 2 to enable full debug
+output.
+
+I<Note that setting the value to 1 will enable the use of regular
+expressions to suppress credential contents. Credentials that do not
+conform to these may still be exposed. Caution is advised.>
+
+
 =item logger
 
 Pass in your own logger that has a C<debug()> method.  Otherwise the
@@ -1039,6 +1119,31 @@ these modules are probably acceptable.
  'constant'                      => '1.33',
  'parent'                        => '0.225',
 
+=head1 SECURITY CONSIDERATIONS
+
+Once credentials are resolved, they will be stored in plain-text
+within this object. The security concern around this fact is not
+actually the fact hat the credentials are in plain-text. Any process
+that compromises your environment can use the same methods this class
+does to resolve those credentials. The concern is that once you have
+resolved these credentials you may inadvertantly explose them in other
+ways. Dumping the object to logs, files or the display will expose
+your credentials.
+
+B<Always take precautions to prevent accidental exfiltration of your
+credentials.>
+
+=over 5
+
+=item * Consider removing them from the hash once they have been
+resolved and used by your application. You can regenerate them using
+the C<find_credentials> method when needed.
+
+=item * Consider using encryption to encrypt the values in memory to
+prevent accidental exposure.
+
+=back
+
 =head1 INCOMPATIBILITIES
 
 This module has not been tested on Windows OS.
